'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); // Required Libraries


var _findRemove = require('find-remove');

var _findRemove2 = _interopRequireDefault(_findRemove);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _durationJs = require('duration-js');

var _durationJs2 = _interopRequireDefault(_durationJs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Create FileProcessor Class

var FileProcessor = function () {

  /**
   * constructor - File processor primary class constructor
   *
   * @param {Boolean} [options.removeAfterProcessed = false] - Should files be able to be deleted
   * @param {String} [options.maxAge = 6h] - Duration string
   */

  function FileProcessor(options) {
    _classCallCheck(this, FileProcessor);

    // Define default options
    this.options = {
      removeFiles: true,
      maxAge: '6h',
      monitorPath: null,
      archivePath: null,
      fileEncoding: 'utf8',
      // chokidar
      persistent: true,
      ignored: /[\/\\]\./,
      ignoreInitial: false,
      followSymlinks: true,
      cwd: '.',
      usePolling: true,
      interval: 100,
      binaryInterval: 300,
      alwaysStat: false,
      depth: 99,
      awaitWriteFinish: {
        stabilityThreshold: 2000,
        pollInterval: 100
      },
      ignorePermissionErrors: false
    };
    // Update options with passed in options
    if (options) {
      _lodash2.default.extend(this.options, options);
    }
    // Get age duration in seconds
    this.duration = new _durationJs2.default(this.options.maxAge).seconds();
  }

  /**
    * Process a file and move it to archive path / remove it
    * @param {String} filePath - The path of the file to process
    * @param {Function} action - File process action... function(fileContents, callback){}
    * @param {String} [archivePath] - If a archive path is provided, the file will be moved to a archive folder instead of being deleted.
    * @param {String} [fileEncoding = utf8]
    * @param {Function} [callback = Function] - Optional callback function, if none is provided a default error log callback will defined.
    */


  _createClass(FileProcessor, [{
    key: 'start',
    value: function start(action) {
      var _this = this;

      console.log('starting monitor');

      // Initialize watcher.
      this.watcher = _chokidar2.default.watch(this.options.monitorPath, this.options);

      // Add event listeners.
      this.watcher.on('add', function (path) {

        _this.constructor.processFile(path, action, _this.options.archivePath, _this.options.fileEncoding, function (err) {
          if (err) return console.log(err);
        });
      });
    }
  }, {
    key: 'stop',
    value: function stop() {
      this.watcher.close();
      delete this.watcher;
      console.log('Watcher stopped');
    }
  }], [{
    key: 'processFile',
    value: function processFile(filePath, action, archivePath, fileEncoding, callback) {

      // Grab file contents and encoding
      var encoding = fileEncoding || "utf8";
      var fileContents = _fsExtra2.default.readFileSync(filePath, encoding);

      // Create empty callback function if none was provided
      if (!callback) {
        callback = function callback(err) {
          if (err) return console.error('ERROR:', err);
        };
      }

      // Execute provided action function with contents and callback function
      action(

      // Pass in file contents to action function
      fileContents,

      // Pass in callback to action function
      function () {

        // Check if a archive path was provided
        if (archivePath) {

          // Make sure the archive path directory is created and ready to use
          _fsExtra2.default.ensureDir(archivePath, function (err) {

            // Check if there were any errors ensuring the directory
            if (err) return callback(err);

            // Create new file date to append file with in archive folder
            var fileDate = (0, _moment2.default)().format('-YYYY-MM-DD-HH-mm');

            // Move file with fs-extra return any errors in callback
            _fsExtra2.default.move(filePath, _path2.default.join(archivePath, _path2.default.basename(filePath) + fileDate), function (err) {
              if (err) return callback(err);
              return callback(null);
            });
          });

          // If a archive path was not provided then remove the processed file
        } else {

            // remove file and return callback with any errors
            _fsExtra2.default.remove(filePath, function (err) {
              if (err) return callback(err);
              return callback(null);
            });
          }
      });
    }

    /**
     * ageFiles - Removes all files in a directory that are x seconds old
     *
     * @param  {String} archivePath   The path of archived files
     * @param  {Number} [ageLimit]    In seconds the max age length per file
     */

  }, {
    key: 'ageFiles',
    value: function ageFiles(archivePath, ageLimit) {
      return (0, _findRemove2.default)(archivePath, {
        age: {
          seconds: ageLimit
        }
      });
    }
  }]);

  return FileProcessor;
}();

// Export FileProcessor class


module.exports = FileProcessor;
//# sourceMappingURL=FileProcessor.js.map
